@@ -26,7 +26,6 @@
 #include <ctype.h>
 #include <assert.h>
 #include <errno.h>
-#include <syslog.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -38,6 +37,7 @@
 #include <net/if.h>
 
 #include "smtpauth.h"
+#include "pam_smtpauth.h"
 
 #ifdef USE_SSL
 #include <openssl/err.h>
@@ -47,9 +47,10 @@
 #include "md5.h"
 #endif
 
-#define EHLO_CMD ("EHLO ")	/* ESMTP ehlo command */
-#define AUTH_CMD ("AUTH ")	/* ESMTP auth command */
-#define QUIT_CMD ("QUIT ")	/* ESMTP quit command */
+#define SMTP_EHLO_CMD "EHLO "	/* ESMTP ehlo command */
+#define SMTP_AUTH_CMD "AUTH "	/* ESMTP auth command */
+#define SMTP_QUIT_CMD "QUIT"	/* ESMTP quit command */
+#define SMTP_NEWLINE  "\r\n"	/* ESMTP newline */
 
 #define RESP_LEN 1000
 #define RESP_IERROR	 "internal error"
@@ -91,6 +92,7 @@ int auth_cram_md5(socket_t *sock, config_t *cfg);
 int auth_digest_md5(socket_t *sock, config_t *cfg);
 
 config_t global;
+param_t params;
 
 static void
 bin2hex(char *out, const unsigned char *in, int in_len) {
@@ -200,7 +202,7 @@ extract_token(const char *str, const char *token, char *value, int len) {
         }
     }
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): str=%s", str);
+    log_debug(DEBUG_9, "extract_token: str=%s", str);
 #endif
 
     return 0;
@@ -223,7 +225,7 @@ digest_md5(char *response, unsigned char *text, unsigned int text_len, const cha
 
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): text=%s", text);
+    log_debug(DEBUG_9, "digest_md5: text=%s", text);
 #endif
 
     extract_token((const char *)text, "nonce=", nonce, DIGEST_MD5_NONCE_LEN);
@@ -237,11 +239,11 @@ digest_md5(char *response, unsigned char *text, unsigned int text_len, const cha
 
     sprintf(uri, "smtp/%s", realm);
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): realm=%s", realm);
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): nonce=%s", nonce);
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): qop=%s", qop);
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): cnonce=%s", cnonce);
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): uri=%s", uri);
+    log_debug(DEBUG_9, "digest_md5: realm=%s", realm);
+    log_debug(DEBUG_9, "digest_md5: nonce=%s", nonce);
+    log_debug(DEBUG_9, "digest_md5: qop=%s", qop);
+    log_debug(DEBUG_9, "digest_md5: cnonce=%s", cnonce);
+    log_debug(DEBUG_9, "digest_md5: uri=%s", uri);
 #endif
 
     /* A1 */
@@ -283,7 +285,7 @@ digest_md5(char *response, unsigned char *text, unsigned int text_len, const cha
     make_digest(hexA1, digest);
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): A1=%s", hexA1);
+    log_debug(DEBUG_9, "digest_md5: A1=%s", hexA1);
 #endif
 
     /* A2 */
@@ -307,7 +309,7 @@ digest_md5(char *response, unsigned char *text, unsigned int text_len, const cha
     make_digest(hexA2, digest);
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): A2=%s", hexA2);
+    log_debug(DEBUG_9, "digest_md5: A2=%s", hexA2);
 #endif
 
     /* response */
@@ -340,7 +342,7 @@ digest_md5(char *response, unsigned char *text, unsigned int text_len, const cha
     make_digest(resp, digest);
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): resp=%s", resp);
+    log_debug(DEBUG_9, "digest_md5: resp=%s", resp);
 #endif
 
     sprintf(response, "charset=utf-8,username=\"%s\",realm=\"%s\",nonce=\"%s\","
@@ -349,8 +351,8 @@ digest_md5(char *response, unsigned char *text, unsigned int text_len, const cha
                login, realm, nonce, cnonce, uri, qop, resp);
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): response:%s", response);
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): text:%s", text);
+    log_debug(DEBUG_9, "digest_md5: response:%s", response);
+    log_debug(DEBUG_9, "digest_md5: text:%s", text);
 #endif
 }
 
@@ -442,9 +444,9 @@ smtp_auth(config_t *cfg) {
                             memset((char*)&taddr, 0, sizeof(taddr));
                             memcpy((char*)&taddr.sin_addr, he->h_addr_list[n], he->h_length);
 #ifdef DEBUG
-                            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): my ip: %s",
+                            log_debug(DEBUG_5, "smtp_auth: my ip: %s",
                               inet_ntoa(((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr));
-                            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): smtp ip: %s",
+                            log_debug(DEBUG_5, "smtp_auth: smtp ip: %s",
                               inet_ntoa(taddr.sin_addr));
 #endif
                             if(((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr.s_addr == taddr.sin_addr.s_addr) {
@@ -471,7 +473,7 @@ smtp_auth(config_t *cfg) {
     }
     if(connect(s, (struct sockaddr *) &addr, sizeof(addr))) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): connection error = %s",strerror(errno));
+        log_debug(DEBUG_1, "smtp_auth: connection error = %s",strerror(errno));
 #endif
         smtp->error = 1;
         strcpy(msgbuf, "Error: connecting to ");
@@ -499,7 +501,7 @@ smtp_auth(config_t *cfg) {
     }
 #endif
 
-    /* CLAIM: we now have a TCP connection to the remote SMTP server */
+    /* TCP connection to the remote SMTP server */
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
     }
@@ -509,7 +511,7 @@ smtp_auth(config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (banner): %m");
+        log_debug(DEBUG_1, "smtp_auth: read (banner): %m");
 #endif
         smtp->error = 1;
         strcpy(msgbuf, RESP_SYNCERROR);
@@ -518,14 +520,14 @@ smtp_auth(config_t *cfg) {
         goto bail;
     }
     rbuf[rc] = '\0';
-    c = strpbrk(rbuf, "\r\n");
+    c = strpbrk(rbuf, SMTP_NEWLINE);
     if(c != NULL) {
         *c = '\0';
     }
 
     if(strncmp(rbuf, "220 ", sizeof("220 ")-1)) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): unexpected response during initial handshake: %s", rbuf);
+        log_debug(DEBUG_1, "smtp_auth: unexpected response during initial handshake: %s", rbuf);
 #endif
         smtp->error = 1;
         strcpy(msgbuf, RESP_UNEXPECTED);
@@ -540,15 +542,15 @@ smtp_auth(config_t *cfg) {
         myhostname = h_name->nodename;
     }
 
-    iov[0].iov_base = EHLO_CMD;
-    iov[0].iov_len  = sizeof(EHLO_CMD) - 1;
+    iov[0].iov_base = SMTP_EHLO_CMD;
+    iov[0].iov_len  = sizeof(SMTP_EHLO_CMD) - 1;
     iov[1].iov_base = myhostname;
     iov[1].iov_len  = strlen(myhostname);
-    iov[2].iov_base = "\r\n";
-    iov[2].iov_len  = sizeof("\r\n") - 1;
+    iov[2].iov_base = SMTP_NEWLINE;
+    iov[2].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s%s", EHLO_CMD, myhostname);
+    log_debug(DEBUG_9, "smtp_auth: sending %s%s", SMTP_EHLO_CMD, myhostname);
 #endif
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
@@ -560,7 +562,7 @@ smtp_auth(config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): writev: %m");
+        log_debug(DEBUG_1, "smtp_auth: writev: %m");
 #endif
         smtp->error = 1;
         strcpy(msgbuf, RESP_IERROR);
@@ -579,7 +581,7 @@ smtp_auth(config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+        log_debug(DEBUG_1, "smtp_auth: read (response): %m");
 #endif
         smtp->error = 1;
         strcpy(msgbuf, RESP_IERROR);
@@ -590,7 +592,7 @@ smtp_auth(config_t *cfg) {
 
     if((tbuf = strstr(rbuf, "250-STARTTLS"))) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): STARTTLS not supported.");
+        log_debug(DEBUG_1, "smtp_auth: STARTTLS not supported.");
 #endif
     }
 
@@ -615,7 +617,7 @@ smtp_auth(config_t *cfg) {
 
     if(avail_auth_type == 0) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): smtp authentication is not implemented: %s", rbuf);
+        log_debug(DEBUG_1, "smtp_auth: smtp authentication is not implemented: %s", rbuf);
 #endif
         smtp->error = 1;
         strcpy(msgbuf, RESP_UNEXPECTED);
@@ -624,7 +626,7 @@ smtp_auth(config_t *cfg) {
         goto bail;
     }
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): auth_type: %d", avail_auth_type);
+    log_debug(DEBUG_1, "smtp_auth: auth_type=%d", avail_auth_type);
 #endif
 
     /* build the AUTH command */
@@ -642,7 +644,7 @@ smtp_auth(config_t *cfg) {
     }
     else {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): smtp authentication is not implemented: %s", rbuf);
+        log_debug(DEBUG_1, "smtp_auth: smtp authentication is not implemented: %s", rbuf);
 #endif
         smtp->error = 1;
         strcpy(msgbuf, RESP_UNEXPECTED);
@@ -652,11 +654,11 @@ smtp_auth(config_t *cfg) {
     }
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth) auth: [%d]", auth);
+    log_debug(DEBUG_5, "smtp_auth: auth=%d", auth);
 #endif
     if(auth == 0) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth) rejected: [%s]", global.username);
+        log_debug(DEBUG_1, "smtp_auth: rejected=%s", global.username);
 #endif
         smtp->error = 2;
         strcpy(msgbuf, RESP_CREDERROR);
@@ -684,13 +686,13 @@ smtp_quit(socket_t *sock, config_t *cfg) {
     struct iovec iov[3];
     int rc;
 
-    iov[0].iov_base = QUIT_CMD;
-    iov[0].iov_len  = sizeof(QUIT_CMD) - 1;
-    iov[1].iov_base = "\r\n";
-    iov[1].iov_len  = sizeof("\r\n") - 1;
+    iov[0].iov_base = SMTP_QUIT_CMD;
+    iov[0].iov_len  = sizeof(SMTP_QUIT_CMD) - 1;
+    iov[1].iov_base = SMTP_NEWLINE;
+    iov[1].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s", QUIT_CMD);
+    log_debug(DEBUG_9, "smtp_quit: sending %s", SMTP_QUIT_CMD);
 #endif
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
@@ -702,7 +704,7 @@ smtp_quit(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): quit writev: %m");
+        log_debug(DEBUG_1, "smtp_quit: quit writev: %m");
 #endif
     }
     (void)socket_close(sock);
@@ -719,17 +721,17 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
     char buf[RESP_LEN];
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): AUTH CRAM-MD5");
+    log_debug(DEBUG_1, "auth_cram_md5: AUTH CRAM-MD5");
 #endif
-    iov[0].iov_base = AUTH_CMD;
-    iov[0].iov_len  = sizeof(AUTH_CMD) - 1;
+    iov[0].iov_base = SMTP_AUTH_CMD;
+    iov[0].iov_len  = sizeof(SMTP_AUTH_CMD) - 1;
     iov[1].iov_base = "CRAM-MD5";
     iov[1].iov_len  = strlen("CRAM-MD5");
-    iov[2].iov_base = "\r\n";
-    iov[2].iov_len  = sizeof("\r\n") - 1;
+    iov[2].iov_base = SMTP_NEWLINE;
+    iov[2].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s%s", AUTH_CMD, "CRAM-MD5");
+    log_debug(DEBUG_9, "auth_cram_md5: sending %s%s", SMTP_AUTH_CMD,"CRAM-MD5");
 #endif
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
@@ -741,7 +743,7 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): cram-md5 writev: %m");
+        log_debug(DEBUG_1, "auth_cram_md5: cram-md5 writev: %m");
 #endif
         return AUTH_NG;
     }
@@ -755,13 +757,13 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+        log_debug(DEBUG_1, "auth_cram_md5: read (response): %m");
 #endif
         return AUTH_NG;
     }
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+    log_debug(DEBUG_5, "auth_cram_md5: read (response): %s",rbuf);
 #endif
     if(strncmp(rbuf, "334 ", sizeof("334 ")-1) == 0) {
         char *response;
@@ -774,7 +776,7 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
         challengelen = base64_decode((char *)challenge, rbuf + 4, -1);
         challenge[challengelen] = '\0';
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): challenge=%s", challenge);
+        log_debug(DEBUG_9, "auth_cram_md5: challenge=%s", challenge);
 #endif
 
         snprintf(buf, sizeof(buf), "%s", cfg->password);
@@ -789,11 +791,11 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
 
         iov[0].iov_base = response64;
         iov[0].iov_len  = strlen(response64);
-        iov[1].iov_base = "\r\n";
-        iov[1].iov_len  = sizeof("\r\n") - 1;
+        iov[1].iov_base = SMTP_NEWLINE;
+        iov[1].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s", response64);
+        log_debug(DEBUG_9, "auth_cram_md5: sending %s", response64);
 #endif
         if(cfg->timeout > 0) {
             set_timeout(cfg->timeout);
@@ -805,7 +807,7 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
         }
         if(rc == -1) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): cram-md5 writev: %m");
+            log_debug(DEBUG_1, "auth_cram_md5: cram-md5 writev: %m");
 #endif
             return AUTH_NG;
         }
@@ -819,24 +821,24 @@ auth_cram_md5(socket_t *sock, config_t *cfg) {
         }
         if(rc == -1) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+            log_debug(DEBUG_1, "auth_cram_md5: read (response): %m");
 #endif
             return AUTH_NG;
         }
 
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+        log_debug(DEBUG_5, "auth_cram_md5: read (response): %s",rbuf);
 #endif
         if(strncmp(rbuf, "235 ", sizeof("235 ")-1) != 0) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): auth failure.");
+            log_debug(DEBUG_1, "auth_cram_md5: auth failure.");
 #endif
             return AUTH_NG;
         }
         free(response64);
     } else {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): it seems cram-md5 mech is not implemented.");
+        log_debug(DEBUG_1, "auth_cram_md5: it seems cram-md5 mech is not implemented.");
 #endif
         return AUTH_NG;
     }
@@ -854,17 +856,17 @@ auth_login(socket_t *sock, config_t *cfg) {
     char *buf;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): AUTH LOGIN");
+    log_debug(DEBUG_1, "auth_login: AUTH LOGIN");
 #endif
-    iov[0].iov_base = AUTH_CMD;
-    iov[0].iov_len  = sizeof(AUTH_CMD) - 1;
+    iov[0].iov_base = SMTP_AUTH_CMD;
+    iov[0].iov_len  = sizeof(SMTP_AUTH_CMD) - 1;
     iov[1].iov_base = "LOGIN";
     iov[1].iov_len  = strlen("LOGIN");
-    iov[2].iov_base = "\r\n";
-    iov[2].iov_len  = sizeof("\r\n") - 1;
+    iov[2].iov_base = SMTP_NEWLINE;
+    iov[2].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s%s", AUTH_CMD, "LOGIN");
+    log_debug(DEBUG_9, "auth_login: sending %s%s", SMTP_AUTH_CMD,"LOGIN");
 #endif
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
@@ -876,7 +878,7 @@ auth_login(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): login writev: %m");
+        log_debug(DEBUG_1, "auth_login: login writev: %m");
 #endif
         return AUTH_NG;
     }
@@ -890,13 +892,13 @@ auth_login(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+        log_debug(DEBUG_1, "auth_login: read (response): %m");
 #endif
         return AUTH_NG;
     }
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+    log_debug(DEBUG_5, "auth_login: read (response): %s",rbuf);
 #endif
     if(strncmp(rbuf, "334 ", sizeof("334 ")-1) == 0) {
         buf = malloc(sizeof(char)*128);
@@ -904,11 +906,11 @@ auth_login(socket_t *sock, config_t *cfg) {
 
         iov[0].iov_base = buf;
         iov[0].iov_len  = strlen(buf);
-        iov[1].iov_base = "\r\n";
-        iov[1].iov_len  = sizeof("\r\n") - 1;
+        iov[1].iov_base = SMTP_NEWLINE;
+        iov[1].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s", buf);
+        log_debug(DEBUG_9, "auth_login: sending %s", buf);
 #endif
         if(cfg->timeout > 0) {
             set_timeout(cfg->timeout);
@@ -920,7 +922,7 @@ auth_login(socket_t *sock, config_t *cfg) {
         }
         if(rc == -1) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): login writev: %m");
+            log_debug(DEBUG_1, "auth_login: login writev: %m");
 #endif
             return AUTH_NG;
         }
@@ -934,13 +936,13 @@ auth_login(socket_t *sock, config_t *cfg) {
         }
         if(rc == -1) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+            log_debug(DEBUG_1, "auth_login: read (response): %m");
 #endif
             return AUTH_NG;
         }
 
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+        log_debug(DEBUG_5, "auth_login: read (response): %s",rbuf);
 #endif
         if(strncmp(rbuf, "334 ", sizeof("334 ")-1) == 0) {
             buf = malloc(sizeof(char)*128);
@@ -948,11 +950,11 @@ auth_login(socket_t *sock, config_t *cfg) {
 
             iov[0].iov_base = buf;
             iov[0].iov_len  = strlen(buf);
-            iov[1].iov_base = "\r\n";
-            iov[1].iov_len  = sizeof("\r\n") - 1;
+            iov[1].iov_base = SMTP_NEWLINE;
+            iov[1].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s", buf);
+            log_debug(DEBUG_9, "auth_login: sending %s", buf);
 #endif
             if(cfg->timeout > 0) {
                 set_timeout(cfg->timeout);
@@ -964,7 +966,7 @@ auth_login(socket_t *sock, config_t *cfg) {
             }
             if(rc == -1) {
 #ifdef DEBUG
-                syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): login writev: %m");
+                log_debug(DEBUG_1, "auth_login: login writev: %m");
 #endif
                 return AUTH_NG;
             }
@@ -978,29 +980,29 @@ auth_login(socket_t *sock, config_t *cfg) {
             }
             if(rc == -1) {
 #ifdef DEBUG
-                syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+                log_debug(DEBUG_1, "auth_login: read (response): %m");
 #endif
                 return AUTH_NG;
             }
 
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+            log_debug(DEBUG_5, "auth_login: read (response): %s",rbuf);
 #endif
             if(strncmp(rbuf, "235 ", sizeof("235 ")-1) != 0) {
 #ifdef DEBUG
-                syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): auth failure.");
+                log_debug(DEBUG_1, "auth_login: auth failure.");
 #endif
                 return AUTH_NG;
             }
         } else {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): it seems login mech is not implemented.");
+            log_debug(DEBUG_1, "auth_login: it seems login mech is not implemented.");
 #endif
             return AUTH_NG;
         }
     } else {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): it seems login mech is not implemented.");
+        log_debug(DEBUG_1, "auth_login: it seems login mech is not implemented.");
 #endif
         return AUTH_NG;
     }
@@ -1020,7 +1022,7 @@ auth_plain(socket_t *sock, config_t *cfg) {
     char phrase[512];
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): AUTH PLAIN");
+    log_debug(DEBUG_1, "auth_plain: AUTH PLAIN");
 #endif
     sprintf(phrase,"%s^%s^%s", cfg->username, cfg->username, cfg->password);
     len = strlen(phrase);
@@ -1032,17 +1034,17 @@ auth_plain(socket_t *sock, config_t *cfg) {
     buf = malloc(sizeof(char)*128);
     base64_encode(buf, phrase, len);
 
-    iov[0].iov_base = AUTH_CMD;
-    iov[0].iov_len  = sizeof(AUTH_CMD) - 1;
+    iov[0].iov_base = SMTP_AUTH_CMD;
+    iov[0].iov_len  = sizeof(SMTP_AUTH_CMD) - 1;
     iov[1].iov_base = "PLAIN ";
     iov[1].iov_len  = strlen("PLAIN ");
     iov[2].iov_base = buf;
     iov[2].iov_len  = strlen(buf);
-    iov[3].iov_base = "\r\n";
-    iov[3].iov_len  = sizeof("\r\n") - 1;
+    iov[3].iov_base = SMTP_NEWLINE;
+    iov[3].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s%s %s", AUTH_CMD, "PLAIN", buf);
+    log_debug(DEBUG_9, "auth_plain: sending %s%s %s", SMTP_AUTH_CMD,"PLAIN",buf);
 #endif
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
@@ -1055,7 +1057,7 @@ auth_plain(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): plain writev: %m");
+        log_debug(DEBUG_1, "auth_plain: plain writev: %m");
 #endif
         return AUTH_NG;
     }
@@ -1069,15 +1071,18 @@ auth_plain(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+        log_debug(DEBUG_1, "auth_plain: read (response): %m");
 #endif
         return AUTH_NG;
     }
 
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+#ifdef DEBUG
+    log_debug(DEBUG_5, "auth_plain: read (response): %s",rbuf);
+#endif
+
     if(strncmp(rbuf, "235 ", sizeof("235 ")-1) != 0) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): auth failure.");
+        log_debug(DEBUG_1, "auth_plain: auth failure.");
 #endif
         return AUTH_NG;
     }
@@ -1094,18 +1099,18 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
     char *buf;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): AUTH DIGEST-MD5");
+    log_debug(DEBUG_1, "auth_digest_md5: AUTH DIGEST-MD5");
 #endif
 
-    iov[0].iov_base = AUTH_CMD;
-    iov[0].iov_len  = sizeof(AUTH_CMD) - 1;
+    iov[0].iov_base = SMTP_AUTH_CMD;
+    iov[0].iov_len  = sizeof(SMTP_AUTH_CMD) - 1;
     iov[1].iov_base = "DIGEST-MD5";
     iov[1].iov_len  = strlen("DIGEST-MD5");
-    iov[2].iov_base = "\r\n";
-    iov[2].iov_len  = sizeof("\r\n") - 1;
+    iov[2].iov_base = SMTP_NEWLINE;
+    iov[2].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s%s", AUTH_CMD, "DIGEST-MD5");
+    log_debug(DEBUG_9, "auth_digest_md5: sending %s%s", SMTP_AUTH_CMD,"DIGEST-MD5");
 #endif
     if(cfg->timeout > 0) {
         set_timeout(cfg->timeout);
@@ -1117,7 +1122,7 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): digest-md5 writev: %m");
+        log_debug(DEBUG_1, "auth_digest_md5: digest-md5 writev: %m");
 #endif
         return AUTH_NG;
     }
@@ -1131,13 +1136,13 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
     }
     if(rc == -1) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+        log_debug(DEBUG_1, "auth_digest_md5: read (response): %m");
 #endif
         return AUTH_NG;
     }
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+    log_debug(DEBUG_5, "auth_digest_md5: read (response): %s",rbuf);
 #endif
     if(strncmp(rbuf, "334 ", sizeof("334 ")-1) == 0) {
         char *response;
@@ -1150,33 +1155,33 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
         challengelen = base64_decode(challenge, rbuf + 4, -1);
         challenge[challengelen] = '\0';
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): challenge=%s", challenge);
+        log_debug(DEBUG_9, "auth_digest_md5: challenge=%s", challenge);
 #endif
 
         digest_md5((char *)hexdigest, (unsigned char*)challenge, challengelen, cfg->username, cfg->password);
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): hexdigest=%s", hexdigest);
+        log_debug(DEBUG_9, "auth_digest_md5: hexdigest=%s", hexdigest);
 #endif
 
         response = malloc(sizeof(char)*256);
         snprintf(response, 256, "%s", hexdigest);
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): response=%s", response);
+        log_debug(DEBUG_9, "auth_digest_md5: response=%s", response);
 #endif
         response64 = malloc((strlen(response) + 3) * 2 + 1);
         base64_encode(response64, response, strlen(response));
         free(response);
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): response64=%s", response64);
+        log_debug(DEBUG_9, "auth_digest_md5: response64=%s", response64);
 #endif
 
         iov[0].iov_base = response64;
         iov[0].iov_len  = strlen(response64);
-        iov[1].iov_base = "\r\n";
-        iov[1].iov_len  = sizeof("\r\n") - 1;
+        iov[1].iov_base = SMTP_NEWLINE;
+        iov[1].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s", response64);
+        log_debug(DEBUG_9, "auth_digest_md5: sending %s", response64);
 #endif
         if(cfg->timeout > 0) {
             set_timeout(cfg->timeout);
@@ -1188,7 +1193,7 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
         }
         if(rc == -1) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): digest-md5 writev: %m");
+            log_debug(DEBUG_1, "auth_digest_md5: digest-md5 writev: %m");
 #endif
             return AUTH_NG;
         }
@@ -1202,13 +1207,13 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
         }
         if(rc == -1) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+            log_debug(DEBUG_1, "auth_digest_md5: read (response): %m");
 #endif
             return AUTH_NG;
         }
 
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+        log_debug(DEBUG_5, "auth_digest_md5: read (response): %s",rbuf);
 #endif
         if(strncmp(rbuf, "334 ", sizeof("334 ")-1) == 0) {
             int buflen;
@@ -1219,11 +1224,11 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
 
             iov[0].iov_base = buf;
             iov[0].iov_len  = strlen(buf);
-            iov[1].iov_base = "\r\n";
-            iov[1].iov_len  = sizeof("\r\n") - 1;
+            iov[1].iov_base = SMTP_NEWLINE;
+            iov[1].iov_len  = sizeof(SMTP_NEWLINE) - 1;
 
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): sending %s", buf);
+            log_debug(DEBUG_9, "auth_digest_md5: sending %s", buf);
 #endif
             if(cfg->timeout > 0) {
                 set_timeout(cfg->timeout);
@@ -1235,7 +1240,7 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
             }
             if(rc == -1) {
 #ifdef DEBUG
-                syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): digest-md5 writev: %m");
+                log_debug(DEBUG_1, "auth_digest_md5: digest-md5 writev: %m");
 #endif
                 return AUTH_NG;
             }
@@ -1249,30 +1254,30 @@ auth_digest_md5(socket_t *sock, config_t *cfg) {
             }
             if(rc == -1) {
 #ifdef DEBUG
-                syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %m");
+                log_debug(DEBUG_1, "auth_digest_md5: read (response): %m");
 #endif
                 return AUTH_NG;
             }
 
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): read (response): %s",rbuf);
+            log_debug(DEBUG_5, "auth_digest_md5: read (response): %s",rbuf);
 #endif
             if(strncmp(rbuf, "235 ", sizeof("235 ")-1) != 0) {
 #ifdef DEBUG
-                syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): auth failure.");
+                log_debug(DEBUG_1, "auth_digest_md5: auth failure.");
 #endif
                 return AUTH_NG;
             }
         } else {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): it seems digest-md5 mech is not implemented.");
+            log_debug(DEBUG_1, "auth_digest_md5: it seems digest-md5 mech is not implemented.");
 #endif
             return AUTH_NG;
         }
         free(response64);
     } else {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): it seems digest-md5 mech is not implemented.");
+        log_debug(DEBUG_1, "auth_digest_md5: it seems digest-md5 mech is not implemented.");
 #endif
         return AUTH_NG;
     }
@@ -1295,7 +1300,7 @@ verify_cert(SSL *ssl) {
     cert = SSL_get_peer_certificate(ssl);
     if(!cert) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): Error: no server certificate.");
+        log_debug(DEBUG_1, "verify_cert: Error: no server certificate.");
 #endif
         return -1;
     }
@@ -1306,8 +1311,7 @@ verify_cert(SSL *ssl) {
     }
 
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): Error:  can't verify certificate: %s (%d).",
-             X509_verify_cert_error_string(err), err);
+    log_debug(DEBUG_1, "verify_cert: Error: can't verify certificate: %s (%d).", X509_verify_cert_error_string(err), err);
 #endif
     X509_NAME_oneline(X509_get_subject_name(cert), buf, sizeof(buf));
     fprintf(stderr,"\nSubject: %s\n", buf);
@@ -1345,7 +1349,7 @@ init_ssl(config_t *conf) {
 
     if(!conf->certfile) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): Error: SSLCertificateFile not defined.");
+        log_debug(DEBUG_1, "init_ssl: Error: SSLCertificateFile not defined.");
 #endif
         return -1;
     }
@@ -1361,16 +1365,16 @@ init_ssl(config_t *conf) {
     if(access(conf->certfile, F_OK)) {
         if(errno != ENOENT) {
 #ifdef DEBUG
-            syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): Error: SSLCertificateFile is not accessible.");
+            log_debug(DEBUG_1, "init_ssl: Error: SSLCertificateFile is not accessible.");
 #endif
             return -1;
         }
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): Warning: SSLCertificateFile doesn't exist, can't verify server certificates.");
+        log_debug(DEBUG_1, "init_ssl: Warning: SSLCertificateFile doesn't exist, can't verify server certificates.");
 #endif
     } else if(!SSL_CTX_load_verify_locations(SSLContext, conf->certfile, NULL)) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): Error: SSL_CTX_load_verify_locations: %s.",ERR_error_string(ERR_get_error(), 0));
+        log_debug(DEBUG_1, "init_ssl: Error: SSL_CTX_load_verify_locations: %s.",ERR_error_string(ERR_get_error(), 0));
 #endif
         SSL_CTX_free(SSLContext);
         return -1;
@@ -1400,7 +1404,7 @@ start_tls(smtp_t *smtp, config_t *cfg) {
     /* initialize SSL */
     if(init_ssl(cfg)) {
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): failed to initialize ssl session.");
+        log_debug(DEBUG_1, "start_tls: failed to initialize ssl session.");
 #endif
         return 1;
     }
@@ -1410,7 +1414,7 @@ start_tls(smtp_t *smtp, config_t *cfg) {
     if((ret = SSL_connect(smtp->sock->ssl)) <= 0) {
         socket_perror("connect", smtp->sock, ret);
 #ifdef DEBUG
-        syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): failed to connect ssl session.");
+        log_debug(DEBUG_1, "start_tls: failed to connect ssl session.");
 #endif
         return 1;
     }
@@ -1422,7 +1426,7 @@ start_tls(smtp_t *smtp, config_t *cfg) {
 #endif
     smtp->sock->use_ssl = 1;
 #ifdef DEBUG
-    syslog(LOG_DEBUG, "pam_smtpauth(smtpauth): SSL support enabled.");
+    log_debug(DEBUG_1, "start_tls: SSL support enabled.");
 #endif
     return 0;
 }
